@@ -28,14 +28,14 @@
 #include "e-cal-backend-exchange-calendar.h"
 
 #include "e2k-cal-utils.h"
-#include "e2k-freebusy.h"
-#include "e2k-propnames.h"
-#include "e2k-restriction.h"
-#include "e2k-utils.h"
-#include "e2k-xml-utils.h"
-#include "e-folder-exchange.h"
-#include "exchange-account.h"
-#include "mapi.h"
+#include <e2k-freebusy.h>
+#include <e2k-propnames.h>
+#include <e2k-restriction.h>
+#include <e2k-utils.h>
+#include <e2k-xml-utils.h>
+#include <e-folder-exchange.h>
+#include <exchange-account.h>
+#include <mapi.h>
 
 struct ECalBackendExchangeCalendarPrivate {
 	int dummy;
@@ -55,6 +55,10 @@ static ECalBackendExchange *parent_class = NULL;
 
 static ECalBackendSyncStatus modify_object_with_href (ECalBackendSync *backend, EDataCal *cal, const char *calobj, CalObjModType mod, char **old_object, const char *href);
 
+static icalproperty *find_attendee_prop (icalcomponent *ical_comp, const char *address);
+static gboolean check_owner_partstatus_for_declined (ECalBackendSync *backend,
+						     icalcomponent *icalcomp);
+
 gboolean check_for_send_options (icalcomponent *icalcomp, E2kProperties *props);
 
 static void
@@ -548,6 +552,61 @@ check_for_send_options (icalcomponent *icalcomp, E2kProperties *props)
 	return exists;
 }
 
+/* stolen from e-itip-control.c with some modifications */
+static icalproperty *
+find_attendee_prop (icalcomponent *ical_comp, const char *address)
+{
+	icalproperty *prop;
+
+	if (address == NULL)
+		return NULL;
+
+	for (prop = icalcomponent_get_first_property (ical_comp, ICAL_ATTENDEE_PROPERTY);
+	     prop != NULL;
+	     prop = icalcomponent_get_next_property (ical_comp, ICAL_ATTENDEE_PROPERTY)) {
+		icalvalue *value;
+		const char *attendee;
+		char *text;
+
+		value = icalproperty_get_value (prop);
+		if (!value)
+			continue;
+
+		attendee = icalvalue_get_string (value);
+
+		if (!g_strncasecmp (attendee, "mailto:", 7))
+			attendee += 7;
+		text = g_strstrip (g_strdup (attendee));
+		if (!g_strcasecmp (address, text)) {
+			g_free (text);
+			return prop;
+		}
+		g_free (text);
+	}
+	
+	return NULL;
+}
+
+static gboolean
+check_owner_partstatus_for_declined (ECalBackendSync *backend,
+				     icalcomponent *icalcomp)
+{
+	icalproperty *icalprop;
+	icalparameter *param;
+	gchar *email;
+	
+	email = e_cal_backend_exchange_get_owner_email (backend);
+	icalprop = find_attendee_prop (icalcomp, email);
+	g_free (email);
+	
+	param = icalproperty_get_first_parameter (icalprop, ICAL_PARTSTAT_PARAMETER);
+	
+	if (icalparameter_get_partstat (param) == ICAL_PARTSTAT_DECLINED) {
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static ECalBackendSyncStatus
 create_object (ECalBackendSync *backend, EDataCal *cal,
 	       char **calobj, char **uid)
@@ -782,13 +841,13 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 
 	/*add object*/
 	e_cal_backend_exchange_add_object (E_CAL_BACKEND_EXCHANGE (cbexc), location, lastmod, icalcomp);
+	*uid = g_strdup (temp_comp_uid);	
 		
 	g_object_unref (comp);
 	g_free (lastmod);
 	g_free (location);
 	e2k_properties_free (props);
 
-	*uid = g_strdup (temp_comp_uid);	
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -1195,6 +1254,10 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 	struct icaltimetype time_rid;
 	ECalBackendSyncStatus ebs_status;
 	
+	/* Will handle only CALOBJ_MOD_THIS and CALOBJ_MOD_ALL for mod.
+	   CALOBJ_MOD_THISANDPRIOR and CALOBJ_MOD_THISANDFUTURE are not
+	   handled. */
+	
 	cbexc = E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
@@ -1214,38 +1277,30 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 	if (old_object)
 		*old_object = e_cal_component_get_as_string (comp);
 	
-	switch (mod) {
+	if (mod == CALOBJ_MOD_THIS && rid && *rid) {
+		/*remove a single instance of a recurring event and modify */
+		time_rid = icaltime_from_string (rid);
+		e_cal_util_remove_instances (ecomp->icomp, time_rid, mod);
+		calobj  = (char *) icalcomponent_as_ical_string (ecomp->icomp);
+		ebs_status = modify_object (backend, cal, calobj, mod, &obj, NULL);
+		g_object_unref (comp);
+		if (ebs_status != GNOME_Evolution_Calendar_Success)
+			goto error;
 		
-		case CALOBJ_MOD_ALL:
-			ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
-			
-			status = e2k_context_delete (ctx, NULL, ecomp->href);			
-			if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {		
-				
-				if (e_cal_backend_exchange_remove_object (E_CAL_BACKEND_EXCHANGE (cbexc), uid)) {
-						g_object_unref (comp);
-						return GNOME_Evolution_Calendar_Success;
-				}
-			}
-			*object = NULL;
-			break;
-		case CALOBJ_MOD_THIS:
-			/*remove_instance and modify */
-			if (rid && *rid) {
-				time_rid = icaltime_from_string (rid);
-				e_cal_util_remove_instances (ecomp->icomp, time_rid, mod);
-			}
-			calobj  = (char *) icalcomponent_as_ical_string (ecomp->icomp);
-			ebs_status = modify_object (backend, cal, calobj, mod, &obj, NULL);
-			if (ebs_status != GNOME_Evolution_Calendar_Success)
-				goto error;
-			
-			g_free (obj);
-			return ebs_status;
-		
-		default:
-			break;
+		g_free (obj);
+		return ebs_status;
 	}
+	g_object_unref (comp);
+
+	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
+	
+	status = e2k_context_delete (ctx, NULL, ecomp->href);
+	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
+		if (e_cal_backend_exchange_remove_object (E_CAL_BACKEND_EXCHANGE (cbexc), uid)) {
+			return GNOME_Evolution_Calendar_Success;
+		}
+	}
+	*object = NULL;
 
 error:
 	return GNOME_Evolution_Calendar_OtherError;
@@ -1300,26 +1355,38 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		case ICAL_METHOD_REQUEST:
 		case ICAL_METHOD_REPLY:
 			if (get_exchange_comp (E_CAL_BACKEND_EXCHANGE (cbexc), uid)) {
-				char *old_object;
+				char *old_object = NULL;
 				
 				calobj = (char *) icalcomponent_as_ical_string (subcomp);
-				status = modify_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object, NULL);
-				if (status != GNOME_Evolution_Calendar_Success)
-					goto error;
-
-				e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, calobj);
+				if (check_owner_partstatus_for_declined (backend, subcomp)) {
+					status = remove_object (backend, cal, uid, NULL, 
+								CALOBJ_MOD_ALL, &old_object,
+								NULL);
+					if (status != GNOME_Evolution_Calendar_Success)
+						goto error;
+					e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid,
+									     old_object, NULL);
+				} else {
+					status = modify_object_with_href (backend, cal, calobj,
+									  CALOBJ_MOD_THIS,
+									  &old_object, NULL);
+					if (status != GNOME_Evolution_Calendar_Success)
+						goto error;
+					e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend),
+									      old_object, calobj);
+				}
 
 				g_free (old_object);
-			} else {
+			} else if (!check_owner_partstatus_for_declined (backend, subcomp)) {
 				char *returned_uid;
-
 				calobj = (char *) icalcomponent_as_ical_string (subcomp);
 				status = create_object (backend, cal, &calobj, &returned_uid);
 				if (status != GNOME_Evolution_Calendar_Success)
 					goto error;
 
 				e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), calobj);
-			}
+			} else
+				status = GNOME_Evolution_Calendar_Success;
 			break;
 		case ICAL_METHOD_ADD:
 			/* FIXME This should be doable once all the recurid stuff is done ??*/
@@ -1368,7 +1435,8 @@ book_resource (ECalBackendExchange *cbex,
 	       EDataCal *cal,
 	       const char *resource_email,
 	       ECalComponent *comp,
-	       icalproperty_method method)
+	       icalproperty_method method,
+	       icalparameter *part_param)
 {
 	E2kGlobalCatalog *gc;
 	E2kGlobalCatalogEntry *entry;
@@ -1386,7 +1454,7 @@ book_resource (ECalBackendExchange *cbex,
 	const char *uid, *prop_name = PR_ACCESS;
 	const char *access_prop = NULL, *meeting_prop = NULL, *cal_uid = NULL;
 	gboolean bookable;
-	char *top_uri = NULL, *cal_uri = NULL, *returned_uid = NULL, *rid = NULL;
+	char *top_uri = NULL, *cal_uri = NULL, *returned_uid = NULL;
 	char *startz, *endz, *href = NULL, *old_object = NULL, *calobj = NULL;
 	E2kRestriction *rn;
 	int nresult;
@@ -1488,12 +1556,16 @@ book_resource (ECalBackendExchange *cbex,
 		e_cal_component_set_summary (E_CAL_COMPONENT (comp), &new_text);
 
 		e_cal_component_set_transparency (E_CAL_COMPONENT (comp), E_CAL_COMPONENT_TRANSP_TRANSPARENT);
-		calobj = (char *) e_cal_component_get_as_string (comp);
-		rid = (char *) e_cal_component_get_recurid_as_string (comp);
-		status = remove_object (E_CAL_BACKEND_SYNC (cbex), cal, uid, rid, CALOBJ_MOD_THIS, &calobj, &object);
-		e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbex), uid, calobj, NULL);
-		g_free (calobj); 
+
+		status = e2k_context_delete (ctx, NULL, href);
+		if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
+			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_OK;
+		} else {
+			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
+		}
+		
 		g_free (object);
+		goto cleanup;
 	} else {
 		/* Check that the new appointment doesn't conflict with any
 		 * existing appointment.
@@ -1555,13 +1627,24 @@ book_resource (ECalBackendExchange *cbex,
 												 
 	/* We're good. Book it. */
 	/* e_cal_component_set_href (comp, href); */
+
+	icalparameter_set_partstat (part_param, ICAL_PARTSTAT_ACCEPTED);
+	
 	e_cal_component_commit_sequence (comp);
 	calobj = (char *) e_cal_component_get_as_string (comp);
 												 
 	/* status = e_cal_component_update (comp, method, FALSE  ); */
 	if (ecomp) {
-		/* This object is already present in the cache so update it. */
+		/* Use the PUT method to create the meeting item in the resource's calendar. */
 		status = modify_object_with_href (E_CAL_BACKEND_SYNC (cbex), cal, calobj, CALOBJ_MOD_THIS, &old_object, href);
+		if (status == GNOME_Evolution_Calendar_Success) {
+			/* Need this to update the participation status of the resource 
+			   in the organizer's calendar. */
+			status = modify_object_with_href (E_CAL_BACKEND_SYNC (cbex), cal, calobj, CALOBJ_MOD_THIS, &old_object, NULL);
+		} else {
+			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
+			goto cleanup;
+		}
 		if (status == GNOME_Evolution_Calendar_Success) {
 			e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbex), old_object, calobj);
 			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_OK;
@@ -1612,11 +1695,11 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 	*modified_calobj = NULL;
 
 	top_level = icalparser_parse_string (calobj);
-	icalcomp = icalcomponent_get_inner (top_level);
+	icalcomp = icalcomponent_new_clone (icalcomponent_get_inner (top_level));
 												 
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (E_CAL_COMPONENT (comp),
-					 icalcomponent_new_clone (icalcomp));
+					   icalcomp);
 												 
 	method = icalcomponent_get_method (top_level);
 	if (icalcomponent_isa (icalcomp) != ICAL_VEVENT_COMPONENT
@@ -1634,7 +1717,7 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 		tzcomp = icalcomponent_get_next_component (top_level,
 							   ICAL_VTIMEZONE_COMPONENT);
 	}
-												 
+
 	for (prop = icalcomponent_get_first_property (icalcomp, ICAL_ATTENDEE_PROPERTY);
 	     prop != NULL;
 	     prop = icalcomponent_get_next_property (icalcomp, ICAL_ATTENDEE_PROPERTY))
@@ -1670,11 +1753,10 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 			goto cleanup;
 		}
 												 
-		result = book_resource (cbex, cal, attendee + 7, comp, method);
+		param = icalproperty_get_first_parameter (prop, ICAL_PARTSTAT_PARAMETER);
+		result = book_resource (cbex, cal, attendee + 7, comp, method, param);
 		switch (result) {
 		case E_CAL_BACKEND_EXCHANGE_BOOKING_OK:
-			param = icalproperty_get_first_parameter (prop, ICAL_PARTSTAT_PARAMETER);
-			icalparameter_set_partstat (param, ICAL_PARTSTAT_ACCEPTED);
 			*users = g_list_append (*users, g_strdup (attendee)) ;
 			break;
 
